from pyfiglet import figlet_format
from typing import List
from colorama import Fore, Style
import calendar
from random import shuffle
import textwrap as tr
import sys
import logging
from sylo.models import Durations, Config
from sylo.utils import get_today, get_weekday, strike


logger = logging.getLogger(__name__)


config = Config()


class Display:
    def __init__(
        self,
        durations: Durations,
        theme_color: str = config.theme_color,
        pom_name: str = config.pom_name,
    ):
        self.theme_color = theme_color
        self.theme = self.color_map(theme_color)
        self.pom_name = pom_name
        self.durations = durations

    def col(self, string: str, color: str = "white"):
        if color:
            return f"{self.color_map(color)}{string}{Style.RESET_ALL}"
        else:
            return f"{self.theme}{string}{Style.RESET_ALL}"

    def print_poms(self, pom: int):
        return f"{self.col(str(pom) + ' ' + self.add_plurality(self.pom_name), 'lmagenta')}"

    def print_timer(self, mode: str, timer_val: int, tasks: List = None):
        if mode == "work":
            self.print_header_small()
            print(
                self.col(f"Tasks for {calendar.day_name[get_weekday()]} {get_today()}")
            )
            print(self.col("-----------------------------------------"))
            self.print_tasks(tasks)
            print(
                f"\n{self.col('Press')} {self.col('Ctrl-C', 'yellow')} "
                f"{self.col('to stop timer early.')}"
                "\n"
                f"\n{self.col('WORK', 'red')} "
                f"{self.col('for')} {self.col(str(timer_val), 'yellow')} {self.col('minutes')}"
            )
        else:
            self.print_header_small()
            print(self.col(f"{calendar.day_name[get_weekday()]} {get_today()}"))
            print(
                f"\n{self.col('Press')} {self.col('Ctrl-C', 'yellow')} "
                f"{self.col('to stop timer early.')}"
                f"\n{self.col('REST', 'green')} "
                f"for {self.col(str(timer_val), 'yellow')} minutes"
            )

    def print_summary_and_quit(self):
        sys.stdout.write("\033[K")
        print(
            f"{self.col('Press')} "
            f"{self.col('ENTER', 'yellow')}"
            f"{self.col('to stop timer.')}",
        )

    def print_insights(self):
        print(
            f"\nTotal {self.col(self.add_plurality(self.pom_name), 'lmagenta')} today:          "
            f"{self.col(str(int(self.durations.poms)), 'yellow')} "
            f"{self.add_plurality(self.pom_name)}"
            f"\nTotal {self.col('Work', 'red')} today:           "
            f"{self.col(str(int(self.durations.total_work_mins)), 'yellow')} minutes"
            f"\nTotal {self.col('Rest', 'green')} today:           "
            f"{self.col(str(int(self.durations.total_rest_mins)), 'yellow')} minutes\n"
            f"\n{self.col('Press')} "
            f"{self.col('ENTER', 'yellow')} "
            f"{self.col('to return to main menu.')}"
        )

    def print_bar_header(self):
        print(
            f"{self.col('Daily work/rest minutes, for the past fortnight', 'yellow')}"
        )

    def print_heat_header(self):
        print(f"{self.col(f'{self.add_plurality(self.pom_name)} heatmap', 'yellow')}")

    def cursor(self):
        return self.col(">> ", "yellow")

    def print_options(self):
        print(
            f"{self.col('Additional commands;')}"
            f"\n{self.col('S', 'yellow')}{self.col('       --    Swap upcoming timer')}"
            f"\n{self.col('T', 'yellow')}{self.col('       --    Show task view')}"
            f"\n{self.col('I', 'yellow')}{self.col('       --    Show insights view')}"
            f"\n{self.col('Q', 'yellow')}{self.col('       --    Quit')}"
        )

    def print_offer_options(self):
        print(f"{self.col('Press')} {self.col('H', 'yellow')} {self.col('for Help.')}")

    def print_add_task_or_quit(self):
        print("\n")
        print(
            f"{self.col('Press')} "
            f"{self.col('ENTER', 'yellow')} "
            f"{self.col('to go back')} "
            f"\n{self.col('Press')} {self.col('N', 'yellow')} "
            f"{self.col('to add a new task')}"
            f"\n{self.col('Press')} {self.col('C', 'yellow')} "
            f"{self.col('to toggle completion of a task')}"
            f"\n{self.col('Press')} {self.col('M', 'yellow')} "
            f"{self.col('to move task to today')}"
            f"\n{self.col('Press')} {self.col('D', 'yellow')} "
            f"{self.col('to delete a task')}"
        )

    def print_new_task(self):
        print(f"{self.col('Input a new task')}")

    def print_edit_task(self):
        print("\n")
        print(f"{self.col('Pick a task to complete/put back')}")

    def print_remove_task(self):
        print("\n")
        print(f"{self.col('Pick a task to DELETE')}")

    def fix_indent(self):
        num = len("pomodoro") - len(self.pom_name)
        fix_str = ""
        for no in range(0, num):
            fix_str += " "
        return fix_str

    def print_splash(self, mode: str, tasks: List):
        print(
            self.col(
                f"Tasks for today, {calendar.day_name[get_weekday()]} {get_today()}"
            )
        )
        print(self.col("-----------------------------------------"))
        self.print_tasks(tasks)

    def print_splash_prompt(self, mode: str):
        print(
            f"\n{self.col('Press')} "
            f"{self.col('ENTER', 'yellow')} "
            f"{self.col('to start the')} "
            f"{self.col(mode.upper(), self._set_mode_color(mode, self.durations))} "
            f"{self.col('timer.')}"
        )

    def print_splash_variants(
        self,
        mode: str,
        is_options: bool = None,
        is_tasks: bool = None,
        tasks: List = None,
    ):
        logger.info(f"Splash. is_options: {is_options} is_tasks: {is_tasks}")
        self.print_header_small()
        self.print_splash(mode, tasks)
        if is_options is True and is_tasks is False:
            self.print_options()
            self.print_splash_prompt(mode)
        elif is_options is False and is_tasks is True:
            self.print_tasks(tasks)
            self.print_splash_prompt(mode)
            self.print_offer_options()
        elif is_options is True and is_tasks is True:
            self.print_tasks(tasks)
            self.print_options()
            self.print_splash_prompt(mode)
        else:
            self.print_splash_prompt(mode)
            self.print_offer_options()

    def print_tasks(self, tasks: List = None):
        complete_tasks = "\n".join(
            "".join(
                map(
                    str,
                    f"{self.col('--')} "
                    f"{self.print_poms(t[3])}: "
                    f"{self.trim_string(self.col(strike(t[2]),'green'),65)}",
                )
            )
            for t in tasks
            if t[4] == True
        )
        incomplete_tasks = "\n".join(
            "".join(
                map(
                    str,
                    f"{self.col('--')} "
                    f"{self.print_poms(t[3])}: "
                    f"{tr.fill(t[2], 45)}",
                )
            )
            for t in tasks
            if t[4] == False
        )
        if complete_tasks:
            print(complete_tasks)
        if incomplete_tasks:
            print(incomplete_tasks)

    def print_incomplete_tasks(self, tasks: List = None):
        incomplete_tasks = "\n".join(
            "".join(
                map(
                    str,
                    f"{self.col('--')} "
                    f"{self.print_poms(t[3])}: "
                    f"{tr.fill(t[2], 45)}",
                )
            )
            for t in tasks
            if t[4] == False
        )
        if incomplete_tasks:
            print(incomplete_tasks)

    def print_tasks_with_id(self, tasks: List = None):
        complete_tasks = "\n".join(
            "".join(
                map(
                    str,
                    f"{self.col('--')} {self.col(t[0], 'yellow')}  "
                    f"{self.print_poms(t[3])}: "
                    f"{self.trim_string(self.col(strike(t[2]),'green'),65)}",
                )
            )
            for t in tasks
            if t[4] == True
        )
        incomplete_tasks = "\n".join(
            "".join(
                map(
                    str,
                    f"{self.col('--')} {self.col(t[0], 'yellow')}  "
                    f"{self.print_poms(t[3])}: "
                    f"{tr.fill(t[2], 45)}",
                )
            )
            for t in tasks
            if t[4] == False
        )
        if complete_tasks:
            print(complete_tasks)
        if incomplete_tasks:
            print(incomplete_tasks)

    def print_header_small(self):
        if self.theme_color != "rainbow":
            print(
                self.col(
                    self.ascii_header(string="SYLO", font="alligator", width=60),
                    self.theme_color,
                )
            )
        else:
            colors = ["lred", "yellow", "lgreen", "lblue", "lmagenta"]
            shuffle(colors)

            # This nasty work-around is because using ascii_header prints each letter to a new row.
            # Forced to add E501 and W291 to flake8 config so remove them if a better solution is found for this.

            print(
                f"""      {self.col('::::::::', colors[0])}  {self.col(':::   :::', colors[1])} {self.col(':::', colors[2])}        {self.col('::::::::', colors[3])} 
    {self.col(':+:    :+:', colors[0])} {self.col(':+:   :+:', colors[1])} {self.col(':+:', colors[2])}       {self.col(':+:    :+:', colors[3])} 
   {self.col('+:+', colors[0])}         {self.col('+:+ +:+', colors[1])}  {self.col('+:+', colors[2])}       {self.col('+:+    +:+', colors[3])}  
  {self.col('+#++:++#++', colors[0])}   {self.col('+#++:', colors[1])}   {self.col('+#+', colors[2])}       {self.col('+#+    +:+', colors[3])}   
        {self.col('+#+', colors[0])}    {self.col('+#+', colors[1])}    {self.col('+#+', colors[2])}       {self.col('+#+    +#+', colors[3])}    
{self.col('#+#    #+#', colors[0])}    {self.col('#+#', colors[1])}    {self.col('#+#', colors[2])}       {self.col('#+#    #+#', colors[3])}     
{self.col('########', colors[0])}     {self.col('###', colors[1])}    {self.col('##########', colors[2])} {self.col('########', colors[3])}                      
"""
            )

    @staticmethod
    def color_map(color: str):
        colors = {
            "red": Fore.RED,
            "lred": Fore.LIGHTRED_EX,
            "blue": Fore.BLUE,
            "lblue": Fore.LIGHTBLUE_EX,
            "green": Fore.GREEN,
            "lgreen": Fore.LIGHTGREEN_EX,
            "yellow": Fore.YELLOW,
            "magenta": Fore.MAGENTA,
            "white": Fore.WHITE,
            "black": Fore.BLACK,
            "cyan": Fore.CYAN,
            "lcyan": Fore.LIGHTCYAN_EX,
            "grey": Fore.LIGHTBLACK_EX,
            "lmagenta": Fore.LIGHTMAGENTA_EX,
            "rainbow": Fore.WHITE,  # Set temporarily to white, set in print_header_small
        }
        return colors[color]

    @staticmethod
    def ascii_header(string: str, font: str, width: int):
        return figlet_format(string, font=font, width=width)

    @staticmethod
    def _set_mode_color(mode: str, durations: Durations):
        if mode == "rest":
            return durations.rest.bar_color
        else:
            return durations.work.bar_color

    @staticmethod
    def add_plurality(string: str):
        if string[-1] == "s":
            return string
        else:
            return string + "s"

    @staticmethod
    def trim_string(string: str, max_len: int) -> str:
        return string[:max_len] + ".." if len(string) > max_len else string



#!/usr/bin/env python
import sys
import logging
import time
import simpleaudio
import csv
from beepy import beep
from tqdm import tqdm
from colorama import Fore
from typing import List
from time import sleep
from sylo.models import (
    Durations,
    Database,
    Config,
    ConfigFile,
)
from sylo.definitions import (
    INPUT_OPTIONS,
    COUNTDOWN_INCREMENTS,
    METRICS_BAR,
    METRICS_HEAT,
    HOME_DIR_CREATE,
    FORTNIGHT_AGO,
    ONE_YEAR_AGO,
    SYLO_HOME,
    INSIGHTS_FILE_SESSIONS,
    INSIGHTS_FILE_POMS,
)
from sylo.utils import (
    clear_screen,
    mins_to_secs,
    flip_mode,
    get_input,
    run_subprocess,
    check_for_dir,
    get_today,
    generate_date_range,
    valid_input,
)
from sylo.display import Display


logger = logging.getLogger(__name__)
is_speed_mode = False
timer_mode = "work"
database = Database()
initial_boot_flag = False


class Timer:
    def __init__(
        self,
        mode: str,
        durations: Durations,
        increments: int,
        config: Config,
        display: Display,
    ):
        self.durations = durations
        self.config = config
        self.mode: str = mode
        self.increments = increments
        self.mins_passed = 0
        self.tasks = database.select(
            f"SELECT"
            f'"id",'
            f'"task_date",'
            f'"description",'
            f'"effort",'
            f'"complete"'
            f"FROM tasks WHERE \"task_date\" = '{get_today()}' "
            f";"
        )
        self.display = display

        if self.mode == "work":
            self.mins = self.durations.work.mins
            self.secs = self.durations.work.secs
            self.bar_color = self.durations.work.bar_color
        else:
            self.mins = self.durations.rest.mins
            self.secs = self.durations.rest.secs
            self.bar_color = self.durations.rest.bar_color

    def _start_timer(self):
        timer_iterator = tqdm(
            total=self.secs,
            position=0,
            leave=False,
            desc=self.display.print_timer(self.mode, self.mins, self.tasks),
            unit=" seconds",
            bar_format="%s{l_bar}%s%s{bar}%s%s{r_bar}%s"
            % (
                self.display.color_map(self.config.theme_color),
                Fore.RESET,
                self.display.color_map(self.bar_color),
                Fore.RESET,
                self.display.color_map(self.config.theme_color),
                Fore.RESET,
            ),
            smoothing=True,
            ncols=70,
        )
        try:
            while self.secs > 0:
                time.sleep(1)
                self.secs -= self.increments
                timer_iterator.update(self.increments)
                if self.secs % 60 == 0:
                    self.mins_passed += 1
                    logger.debug(f"{self.mode} Timer loop: secs remaining: {self.secs}")

        except KeyboardInterrupt:
            print("\nQuitting early..")
            logger.info(f"Ctrl-C quit with {self.secs} remaining")
            timer_iterator.close()
            return False, self.mins_passed
            pass
        logger.info(f"{self.mode} timer loop finished")
        timer_iterator.close()
        return True, self.mins_passed

    def start_countdown(self):
        logger.info(f"{self.mode} timer started")
        complete, mins_passed = self._start_timer()
        return complete, mins_passed


class Sound:
    def __init__(self, audio_path: str = "dummy"):
        self.sound_file = audio_path
        self.beep_needed = False
        self.initialised_file = None

    def initialise(self):
        logger.info(f"Initialising audio: {self.sound_file}")
        try:
            self.initialised_file = simpleaudio.WaveObject.from_wave_file(
                self.sound_file
            )
            logger.info(f"Custom audio successfully initialised: {self.sound_file}")
        except FileNotFoundError:
            logger.info(f"Custom audio not found: {self.sound_file}")
            self.beep_needed = True
        except AttributeError:
            logger.info(f"Custom audio not found: {self.sound_file}")
            self.beep_needed = True

    def play_sound(self):
        if self.beep_needed is False:
            play_obj = self.initialised_file.play()
            logger.info("Played audio file")
            play_obj.wait_done()
            logger.info("Waiting audio file")

        else:
            logger.info("Playing beep")
            beep("ready")


class Insights:
    def __init__(self, display: Display):
        self.data_file = INSIGHTS_FILE_SESSIONS
        self.poms_file = INSIGHTS_FILE_POMS
        self.heatmap_start_date = ONE_YEAR_AGO
        self.bar_start_date = FORTNIGHT_AGO
        self.display = display

    def save_files(self):
        csv_writer = csv.writer(open(self.data_file, "w"))
        data = database.select(
            f""
            f"SELECT * "
            f"FROM sessions "
            f"WHERE session_date >= {self.bar_start_date}"
        )
        logger.info(data)
        for row in data:
            csv_writer.writerow(row)

        csv_writer = csv.writer(open(self.poms_file, "w"))
        data = database.select(
            f""
            f"SELECT * "
            f"FROM poms "
            f"WHERE poms_date >= {self.heatmap_start_date}"
        )
        logger.info(data)
        for row in data:
            csv_writer.writerow(row)

    def print_bar(self):
        global initial_boot_flag
        self.display.print_header_small()
        self.display.print_bar_header()
        if initial_boot_flag is False:
            logger.info(f"Running ${self._bar_chart()}")
            run_subprocess(self._bar_chart())
        else:
            print("No data available, complete some segments and check later.")

    def print_heat(self):
        logger.info(f"Running ${self._heat_map()}")
        self.display.print_heat_header()
        run_subprocess(self._heat_map())

    def _bar_chart(self):
        return METRICS_BAR % (self.data_file, self.bar_start_date)

    def _heat_map(self):
        return METRICS_HEAT % (self.heatmap_start_date, self.poms_file)


class Task:
    def __init__(self):
        self.description = None
        self.date = get_today()
        self.complete = 0
        self.effort = 1

    def write_task_to_db(self):
        database.execute(
            f"INSERT INTO tasks ("
            f'"task_date", '
            f'"description", '
            f'"effort", '
            f'"complete"'
            f") VALUES("
            f'"{self.date}",'
            f'"{self.description}",'
            f'"{self.effort}",'
            f'"{self.complete}");'
        )


class Sylo:
    def __init__(
        self,
        durations: Durations,
        sound_obj: any,
        mode: str,
        show_options: bool = False,
        show_tasks: bool = False,
        tasks: List = None,
        tasks_date: str = None,
        speed_mode: bool = is_speed_mode,
        config: Config = None,
        display: Display = None,
    ):
        self.durations = durations
        self.sound_obj = sound_obj
        self.mode = mode
        self.show_options = show_options
        self.show_tasks = show_tasks
        self.tasks = tasks
        self.tasks_date = tasks_date
        self.speed_mode = speed_mode
        self.increments = 1
        self.config = config
        self.display = display

    def on_boot(self):
        clear_screen()
        if self.speed_mode is True:
            self.increments = 30
        else:
            self.increments = COUNTDOWN_INCREMENTS
        logger.debug(f"Running in {self.increments} second increments")

    def splash(self):
        self.display.print_splash_variants(
            mode=self.mode,
            is_options=self.show_options,
            is_tasks=self.show_tasks,
            tasks=self.tasks,
        )

    def show_task(self, dates, with_id: bool = False):
        for date in dates:
            print(self.display.col(f"---------------------------- {date}"))
            self.tasks = database.select(
                f"SELECT"
                f'"id",'
                f'"task_date",'
                f'"description",'
                f'"effort",'
                f'"complete"'
                f"FROM tasks WHERE \"task_date\" = '{date}'"
                f";"
            )
            if with_id is False:
                self.display.print_tasks(self.tasks)
            else:
                self.display.print_tasks_with_id(self.tasks)
            logger.info(f"Tasks: {self.tasks}")

    def task_menu_new(self, dates: List):
        task = Task()
        self.display.print_new_task()
        task.description = input(self.display.col("Description >> ", "yellow"))
        task.effort = input(
            self.display.col(
                f"Effort (in"
                f" {self.display.add_plurality(self.display.pom_name)}) >> ",
                "yellow",
            )
        )
        while task.effort.isnumeric() is False:
            print(self.display.col("MUST BE NUMERIC", "red"))
            sleep(1)
            self.task_menu(dates)
        task.write_task_to_db()
        self.task_menu(dates)

    def task_menu_close(self, dates: List, task_ids: List[tuple]):
        clear_screen()
        self.display.print_header_small()
        self.show_task(dates, True)
        self.display.print_edit_task()
        comp_input = input(self.display.col("Task id >> ", "yellow"))
        while valid_input(comp_input, task_ids) is False:
            print(self.display.col("NOT A VALID ID", "red"))
            sleep(1)
            self.task_menu(dates)
        database.execute(
            f'UPDATE tasks SET "complete" = '
            f'CASE WHEN "complete" = False THEN True ELSE False END'
            f' WHERE "id" = {comp_input};'
        )
        self.task_menu(dates)

    def task_menu_move(self, dates: List, task_ids: List[tuple]):
        clear_screen()
        self.display.print_header_small()
        self.show_task(dates, True)
        self.display.print_edit_task()
        comp_input = input(self.display.col("Task id >> ", "yellow"))
        while valid_input(comp_input, task_ids) is False:
            print(self.display.col("NOT A VALID ID", "red"))
            sleep(1)
            self.task_menu(dates)
        database.execute(
            f"UPDATE tasks SET \"task_date\" = '{get_today()}'"
            f' WHERE "id" = {comp_input};'
        )
        self.task_menu(dates)

    def task_menu_delete(self, dates: List, task_ids: List[tuple]):
        clear_screen()
        self.display.print_header_small()
        self.show_task(dates, True)
        self.display.print_remove_task()
        comp_input = input(self.display.col("Task id >> ", "yellow"))
        while valid_input(comp_input, task_ids) is False:
            print(self.display.col("NOT A VALID ID", "red"))
            sleep(1)
            self.task_menu(dates)
        database.execute(f"DELETE FROM tasks" f' WHERE "id" = {comp_input};')
        self.task_menu(dates)

    def task_menu(self, dates: List):
        clear_screen()
        all_tasks = self.tasks = database.select(
            f"SELECT"
            f'"id",'
            f'"task_date",'
            f'"description",'
            f'"effort",'
            f'"complete"'
            f'FROM tasks WHERE "task_date" '
            f">= (SELECT DATETIME({(get_today())}, '-7 day'))"
            f";"
        )
        all_task_ids = [i[0] for i in all_tasks]
        self.display.print_header_small()
        self.show_task(dates)
        self.display.print_add_task_or_quit()
        usr_input = input(self.display.cursor())
        while valid_input(usr_input, INPUT_OPTIONS["task_menu"]) is False:
            self.task_menu(dates)
        if usr_input == "":
            self.home_loop()
        while usr_input.lower() == "n":
            self.task_menu_new(dates)
        while usr_input.lower() == "c":
            self.task_menu_close(dates, all_task_ids)
        while usr_input.lower() == "m":
            self.task_menu_move(dates, all_task_ids)
        while usr_input.lower() == "d":
            self.task_menu_delete(dates, all_task_ids)

    def switch_modes(self):
        logger.info("Switch requested")
        self.mode = flip_mode(self.mode)
        self.home_loop()

    def show_help_menu(self):
        logger.info("Help requested")
        self.show_options ^= True
        self.home_loop()

    def show_insights_menu(self):
        clear_screen()
        metrics = Insights(self.display)
        metrics.save_files()
        metrics.print_bar()
        metrics.print_heat()
        self.display.print_insights()
        input(self.display.cursor())
        self.home_loop()

    def timer_loop(self, timer: Timer):
        clear_screen()
        global initial_boot_flag
        logger.info("User pressed ENTER")
        if self.mode == "rest":
            complete, secs_passed = timer.start_countdown()
            if complete is True:
                self.sound_obj.play_sound()
                self.durations.total_rest_mins += secs_passed
            else:
                self.durations.total_rest_mins += secs_passed
            self.mode = flip_mode(self.mode)
            database.execute(
                'INSERT INTO sessions ("session_date", "time_worked", "time_rested")'
                f"VALUES ('{get_today()}',"
                f"'{self.durations.total_work_mins}', "
                f"'{self.durations.total_rest_mins}')"
                'ON CONFLICT ("session_date") DO UPDATE SET '
                '("time_worked", "time_rested") = (excluded."time_worked", excluded."time_rested")'
            )
            clear_screen()
        else:
            complete, secs_passed = timer.start_countdown()
            if complete is True:
                self.sound_obj.play_sound()
                self.durations.total_work_mins += self.durations.work.mins
                self.durations.poms += 1
            else:
                self.durations.total_work_mins += secs_passed
            self.mode = flip_mode(self.mode)
            database.execute(
                'INSERT INTO sessions ("session_date", "time_worked", "time_rested")'
                f"VALUES ('{get_today()}',"
                f"'{self.durations.total_work_mins}', "
                f"'{self.durations.total_rest_mins}')"
                'ON CONFLICT ("session_date") DO UPDATE SET '
                '("time_worked", "time_rested") = (excluded."time_worked", excluded."time_rested")'
            )
            database.execute(
                'INSERT INTO poms ("poms_date", "poms")'
                f"VALUES ('{get_today()}',"
                f"'{self.durations.poms}')"
                'ON CONFLICT ("poms_date") DO UPDATE SET '
                '"poms" = excluded."poms"'
            )
            clear_screen()
        logger.info("Durations model set")
        logger.info(f"Work time: {self.durations.total_work_mins}")
        logger.info(f"Work time: {self.durations.total_rest_mins}")
        initial_boot_flag = False
        self.home_loop()

    @staticmethod
    def quit_app():
        logger.info("Quit requested")
        clear_screen()
        sys.exit()

    def home_loop(self):
        self.on_boot()
        timer = Timer(
            mode=self.mode,
            durations=self.durations,
            config=self.config,
            increments=self.increments,
            display=self.display,
        )
        self.splash()
        response = get_input(self.display.cursor())
        logger.info(f"User response: {response}")
        while valid_input(response, INPUT_OPTIONS["home"]) is False:
            self.show_options = True
            logger.info(f"Rejected user response of {response}")
            self.home_loop()
        while response.lower() == "q":
            self.quit_app()
        while response.lower() == "i":
            self.show_insights_menu()
        while response.lower() == "s":
            self.switch_modes()
        while response.lower() == "h":
            self.show_help_menu()
        while response.lower() == "t":
            self.task_menu(generate_date_range(get_today(), 7))
        while response == "":
            self.timer_loop(timer)


def virgin_boostrap():
    global initial_boot_flag
    initial_boot_flag = True
    logger.info("First boot")
    run_subprocess(HOME_DIR_CREATE)


def config_allocation(args, config, durations: Durations, display: Display):
    config_file = ConfigFile()
    config_file.set_config(config, durations)
    logger.info(config.theme_color)
    if args.theme:
        config.theme_color = args.theme
    logger.info(config.theme_color)
    display.theme_color = config.theme_color
    display.theme = display.color_map(config.theme_color)
    display.pom_name = config.pom_name
    logger.info(display.theme)
    if args.audio_file:
        config.audio_file = args.audio_file
    if args.work_time:
        durations.work.mins = args.work_time
        durations.work.secs = mins_to_secs(args.work_time)
    if args.rest_time:
        durations.rest.mins = args.rest_time
        durations.rest.secs = mins_to_secs(args.rest_time)


def run(args, config):
    global is_speed_mode, timer_mode

    durations_data = Durations()
    display = Display(durations=durations_data)
    config_allocation(args, config, durations_data, display)
    sound = Sound(config.audio_file)
    sound.initialise()

    if check_for_dir(SYLO_HOME) is False:
        virgin_boostrap()
    clear_screen()
    database.bootstrap_on_first_boot()
    database.initial_data_load(durations_data)

    logger.info("Durations model set")
    logger.info(f"Work time:    {durations_data.work.mins}")
    logger.info(f"Work time:    {durations_data.work.secs}")
    logger.info(f"Work time:    {durations_data.rest.mins}")
    logger.info(f"Work time:    {durations_data.rest.secs}")
    logger.info(f"Sound file:   {sound.sound_file}")

    if args.speed_mode is True:
        is_speed_mode = True
    logger.info(f"Speed mode set to {is_speed_mode}")

    sylo = Sylo(
        durations=durations_data,
        sound_obj=sound,
        mode=timer_mode,
        speed_mode=is_speed_mode,
        tasks=database.select(
            f"SELECT"
            f'"id",'
            f'"task_date",'
            f'"description",'
            f'"effort",'
            f'"complete"'
            f"FROM tasks "
            f"WHERE \"task_date\" = '{get_today()}'"
            f";"
        ),
        config=config,
        display=display,
    )
    sylo.home_loop()



import os
import datetime
from pathlib import Path


HOME_DIR = home = str(Path.home())
ROOT_DIR = os.path.dirname(os.path.abspath(__file__))

CONFIG_FILE_LOCATION = f"{HOME_DIR}/.sylo/sylo.cfg"
HOME_DIR_CREATE = f"mkdir {HOME_DIR}/.sylo"
SYLO_HOME = f"{HOME_DIR}/.sylo"


INPUT_OPTIONS = {
    "home": ["", "h", "q", "s", "t", "y", "i", "a"],
    "task_menu": ["", "c", "m", "n", "d"],
}


TIMER_DEFAULTS = {
    "work": {
        "mins": 25,
        "secs": 1500,
    },
    "rest": {
        "mins": 5,
        "secs": 300,
    },
}

COUNTDOWN_INCREMENTS = 1

METRICS_HEAT = 'termgraph --calendar --color magenta --start-dt "%s" %s 2>/dev/null'

METRICS_BAR = 'termgraph %s --color {red,green} --start-dt "%s" 2>/dev/null'

SQLITE_DB = f"{HOME_DIR}/.sylo/_sylo.db"

BOOTSTRAP_SQL = f"{ROOT_DIR}/database_interactions/bootstrap.sql"

INSIGHTS_FILE_SESSIONS = f"{HOME_DIR}/.sylo/insights_sessions.dat"
INSIGHTS_FILE_POMS = f"{HOME_DIR}/.sylo/insights_poms.dat"
FORTNIGHT_AGO = (datetime.datetime.now() - datetime.timedelta(days=14)).date()
ONE_YEAR_AGO = (datetime.datetime.now() - datetime.timedelta(days=364)).date()



import os
import logging
import subprocess
import sys
from typing import List
from datetime import datetime, timedelta


logger = logging.getLogger(__name__)


def get_input(text):
    return input(text)


def valid_input(text, allowed_list: List):
    logger.info(f"Validating user choice against list: {allowed_list}")
    allowed_list_strings = [str(al) for al in allowed_list]
    if text.lower() in allowed_list_strings:
        return True
    else:
        return False


def run_subprocess(script: str):
    if "rm " in script:
        logger.warning("UNSAFE SCRIPT")
        sys.exit()
    try:
        subprocess.run(script, shell=True, check=True, timeout=30)
    except subprocess.CalledProcessError:
        print(
            "Insights panel not loading as there is no data to display. "
            "\nTry working for a few mins then check back later."
        )
        print("\n")


def clear_screen():
    os.system("cls" if os.name == "nt" else "clear")


def mins_to_secs(minutes: int):
    return minutes * 60


def flip_mode(mode: str):
    if mode == "work":
        logger.debug(f"flip_mode took {mode} returned rest")
        return "rest"
    else:
        logger.debug(f"flip_mode took {mode} returned work")
        return "work"


def check_for_file(path):
    is_file = os.path.isfile(path)
    logger.info(f"check_for_file {path} {is_file}")
    return is_file


def check_for_dir(path):
    is_dir = os.path.isdir(path)
    logger.info(f"check_for_file {path} {is_dir}")
    return is_dir


def get_today():
    return datetime.now().date().strftime("%Y-%m-%d")


def get_weekday():
    return datetime.now().weekday()


def read_from_file(path):
    with open(path, "r") as read_file:
        return read_file.read()


def generate_date_range(end_date: str, number_of_days: int):
    end_date_dt = datetime.strptime(end_date, "%Y-%m-%d")
    delta = timedelta(days=1)
    date_list = []
    for day in range(number_of_days):
        date_list.append(datetime.strftime(end_date_dt, "%Y-%m-%d"))
        end_date_dt -= delta
    date_list.sort()
    return date_list


def strike(text):
    result = ""
    for c in text:
        result = result + c + "\u0336"
    return result



import logging
from dataclasses import dataclass
from sylo.definitions import (
    TIMER_DEFAULTS,
    CONFIG_FILE_LOCATION,
    SQLITE_DB,
)
from sylo.utils import check_for_file, mins_to_secs, get_today
import tomli
import sqlite3


logger = logging.getLogger(__name__)


@dataclass
class Rest:
    mins: int = TIMER_DEFAULTS["rest"]["mins"]
    secs: int = TIMER_DEFAULTS["rest"]["secs"]
    bar_color: str = "green"


@dataclass
class Work:
    mins: int = TIMER_DEFAULTS["work"]["mins"]
    secs: int = TIMER_DEFAULTS["work"]["secs"]
    bar_color: str = "red"


@dataclass
class Durations:
    work = Work
    rest = Rest
    total_work_mins: float = 0
    total_rest_mins: float = 0
    poms = float = 0

    def __post_init__(self):
        self.work = Work()
        self.rest = Rest()


@dataclass
class Config:
    theme_color = "lblue"
    audio_file = None
    work_time = 25
    rest_time = 5
    pom_name = "Pomodoro"


class ConfigFile:
    def __init__(self):
        self.config_file = CONFIG_FILE_LOCATION
        self.is_config = check_for_file(self.config_file)

    def load_config(self):
        with open(self.config_file, encoding="utf-8") as f:
            return tomli.load(f)

    def set_config(self, config: Config, durations_model: Durations):
        if self.is_config is True:
            config_file = self.load_config()
            logger.info(config_file)
            try:
                general = config_file["general"]
                logger.info(f"CONFIG: general from file: {general}")
                config.theme_color = general["theme"]
                logger.info(f"CONFIG:set theme_color to {config.theme_color}")

            except KeyError as k:
                logger.info(k)
                pass
            try:
                general = config_file["general"]
                config.audio_file = general["audio_file"]
                logger.info(f"CONFIG:set audio_file to {config.audio_file}")
            except KeyError as k:
                logger.info(k)
                pass
            try:
                general = config_file["general"]
                config.pom_name = general["time_segment_name"]
                logger.info(f"CONFIG:set pom_name to {config.pom_name}")
            except KeyError as k:
                logger.info(k)
                pass

            try:
                durations = config_file["durations"]
                logger.info(f"CONFIG: durations from file: {durations}")
                durations_model.work.mins = durations["work"]
                durations_model.work.secs = mins_to_secs(durations["work"])
                durations_model.rest.mins = durations["rest"]
                durations_model.rest.secs = mins_to_secs(durations["rest"])
            except KeyError as k:
                logger.info(k)
                pass


class Database:
    def __init__(self):
        self.db_file = SQLITE_DB

    def _connect(self):
        return sqlite3.connect(self.db_file)

    def execute(self, query: str):
        con = self._connect()
        cur = con.cursor()
        cur.execute(query)
        con.commit()
        con.close()

    def execute_script(self, query: str):
        con = self._connect()
        cur = con.cursor()
        cur.executescript(query)
        con.commit()
        con.close()

    def select(self, query: str):
        con = self._connect()
        cur = con.cursor()
        result = cur.execute(query)
        data = result.fetchall()
        cur.close()
        return data

    def execute_from_file(self, file: str):
        with open(file, "r") as read_file:
            query = read_file.read()
        self.execute_script(query)

    def select_from_file(self, file: str):
        with open(file, "r") as read_file:
            query = read_file.read()
        return self.select(query)

    def bootstrap_on_first_boot(self):
        self.execute_script(bootstrap())

    def initial_data_load(self, durations: Durations):
        (session_data,) = (
            self.select(
                f"SELECT * FROM sessions where session_date = '{get_today()}';"
            ),
        )
        (poms_data,) = (
            self.select(f"SELECT * FROM poms where poms_date = '{get_today()}';"),
        )
        try:
            logger.info(session_data)
            work_time_today = session_data[0][1]
            rest_time_today = session_data[0][2]
            logger.info(f"Work time loaded on boot: {work_time_today}")
            logger.info(f"Rest time loaded on boot: {rest_time_today}")
            durations.total_work_mins = work_time_today
            durations.total_rest_mins = rest_time_today
            logger.info("data_load_on_boot Found data for today")
        except Exception as e:
            logger.info(e)
            logger.info("data_load_on_boot NO DATA for today")
        if session_data is None:
            durations.total_rest_mins = 0
            durations.total_work_mins = 0

        try:
            logger.info(poms_data)
            poms_today = poms_data[0][1]
            logger.info(f"Poms loaded on boot: {poms_today}")
            durations.poms = poms_today
            logger.info("data_load_on_boot Found poms for today")
        except Exception as e:
            logger.info(e)
            logger.info("data_load_on_boot NO DATA for today")
        if poms_data is None:
            durations.poms = 0


def bootstrap():
    return """
    CREATE TABLE IF NOT EXISTS sessions (
        session_date date,
        time_worked REAL,
        time_rested REAL,
        UNIQUE(session_date)
    );

    CREATE TABLE IF NOT EXISTS poms (
        poms_date date,
        poms int,
        UNIQUE(poms_date)
        );

    CREATE TABLE IF NOT EXISTS tasks (
        id INTEGER PRIMARY KEY,
        task_date date,
        description text,
        effort int,
        complete bool
    );
    """
